/*
 * randpass.c --- generates a random password, using MD5 as the random
 * 	number generator
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>

#include "hmac_md5.h"

#define MAX_SECRET_LENGTH  4096

static HMAC_MD5_CTX randpass_ctx;

static char randpass_tab[] =
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_$";

int randpass_init(char *file_name)
{
	int		fd;
	int		count;
	unsigned char	buf[MAX_SECRET_LENGTH];

	fd = open(file_name, O_RDONLY);
	if (fd < 0)
		return errno;

	count = read(fd, buf, sizeof(buf));
	close(fd);
	if (count < 0)
		return errno;
	HMAC_MD5_SetKey(&randpass_ctx, buf, count);
	return 0;
}

struct tbuf {
  struct timeval tv;
  int count;
};

int randpass_gen(char *password, int length)
{	
	struct tbuf tbuf;
	static int count = 0;
	int		i, b, ret, ch;
	unsigned char	digest[16];

	ret = gettimeofday(&tbuf.tv, 0);
	if (ret < 0)
		return errno;
	tbuf.count = count++;

	HMAC_MD5(&randpass_ctx, (unsigned char *) &tbuf, sizeof(tbuf), digest);

	for (i=0, b=0; i < length; i++) {
	retry:
		password[i] = ch = randpass_tab[digest[b] % 64];
		if (b++ == sizeof(digest))
			b = 0;
		if (i==0 && !(isalpha(ch)))
			goto retry;
	}
	password[length] = 0;
	return 0;
}

#ifdef TEST
int main(int argc, char **argv)
{
	int	i;
	char	password[9];
	
	randpass_init(TEST);
	for (i=0; i < 8; i++) {
		randpass_gen(password, 8);
		printf("Password %d: %s\n", i, password);
	}
	return 0;
}
#endif
