/* 
 * WinSock support.
 *
 * Do the WinSock initialization call, keeping all the hair here.
 *
 * This routine is called by SOCKET_INITIALIZE in include/c-windows.h.
 * The code is pretty much copied from winsock.txt from winsock-1.1,
 * available from:
 * ftp://sunsite.unc.edu/pub/micro/pc-stuff/ms-windows/winsock/winsock-1.1
 *
 * Note: WSAStartup and WSACleanup is called here (and only here).
 * This assumes that under Windows, we only use this library via the
 * DLL.  Note that calls to WSAStartup and WSACleanup must be in
 * matched pairs.  If there is a missing WSACleanup call when a
 * program exits, under Lan Workplace, the name resolver will stop
 * working. 
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "gss-client.h"

__declspec(dllexport) int __stdcall
kludge_get_connect_info2(char *host, char *service, char *sid, char *port,
			 char username[32], char password[32],
			 char error_string[128])
{
	char *u, *p, *e;
	char sname[256];
	unsigned int portnum;

	sprintf(sname, "%s@%s", service ? service: "host", host);

	if (sid && (sid[0] == 0))
		sid = 0;

	if (port && port[0])
		portnum = atoi(port);
	else
		portnum = 17007;

	call_kludge_server(host, portnum, sname, sid, &u, &p, &e);
	if (u) {
		strncpy(username, u, 32);
		username[31] = 0;
		free(u);
	} else
		username[0] = 0;
	if (p) {
		strncpy(password, p, 32);
		password[31] = 0;
		free(p);
	} else
		password[0] = 0;
	if (e) {
		strncpy(error_string, e, 128);
		error_string[127] = 0;
		free(e);
	} else
		error_string[0] = 0;
	return 0;
}

__declspec(dllexport) int __stdcall
kludge_get_connect_info(host, username, password, error_string)
     char *host;
     char username[32];
     char password[32];
     char error_string[128];
{
     return kludge_get_connect_info2(host, 0, 0, 0, username,
				     password, error_string);
}


static HINSTANCE hlibinstance;

HINSTANCE get_lib_instance()
{
    return hlibinstance;
}

#define DLL_STARTUP 0
#define DLL_SHUTDOWN 1

static int
control(int mode)
{
#ifdef NEED_WINSOCK
    WORD wVersionRequested;
    WSADATA wsaData;
    int err;
#endif

    switch(mode) {
    case DLL_STARTUP:

#ifdef NEED_WINSOCK
	wVersionRequested = 0x0101;		/* We need version 1.1 */
	if ((err = WSAStartup (wVersionRequested, &wsaData)))
	    return err;
	if (wVersionRequested != wsaData.wVersion) {
	    /* DLL couldn't support our version of the spec */
	    WSACleanup ();
	    return -104;			/* FIXME -- better error? */
	}
#endif

	break;

    case DLL_SHUTDOWN:
#ifdef NEED_WINSOCK
	WSACleanup ();
#endif
	break;

    default:
	return -1;
    }

    return 0;
}

#ifdef _WIN32

BOOL WINAPI DllMain (HANDLE hModule, DWORD fdwReason, LPVOID lpReserved)
{
    switch (fdwReason)
    {
        case DLL_PROCESS_ATTACH:
	    hlibinstance = (HINSTANCE) hModule;
	    if (control(DLL_STARTUP))
		return FALSE;
	    break;

        case DLL_THREAD_ATTACH:
	    break;

        case DLL_THREAD_DETACH:
	    break;

        case DLL_PROCESS_DETACH:
	    if (control(DLL_SHUTDOWN))
		return FALSE;
	    break;

        default:
	    return FALSE;
    }
 
    return TRUE;   // successful DLL_PROCESS_ATTACH
}

#else

BOOL CALLBACK
LibMain (hInst, wDataSeg, cbHeap, CmdLine)
HINSTANCE hInst;
WORD wDataSeg;
WORD cbHeap;
LPSTR CmdLine;
{
    hlibinstance = hInst;
    if (control(DLL_STARTUP))
	return 0;
    else 
	return 1;
}

int CALLBACK __export
WEP(nParam)
	int nParam;
{
    if (control(DLL_SHUTDOWN))
	return 0;
    else
	return 1;
}

#endif
